@@ -11,7 +11,9 @@ from sql import Literal, Union, Column, Null, For
 from sql.aggregate import Sum
 from sql.conditionals import Coalesce, Case
 
+from trytond.i18n import gettext
 from trytond.model import Workflow, Model, ModelView, ModelSQL, fields, Check
+from trytond.model.exceptions import AccessError
 from trytond.pyson import Eval, If, Bool
 from trytond.tools import reduce_ids
 from trytond.transaction import Transaction
@@ -19,6 +21,8 @@ from trytond.pool import Pool
 
 from trytond.modules.product import price_digits
 
+from .exceptions import MoveOriginWarning
+
 __all__ = ['StockMixin', 'Move']
 
 STATES = {
@@ -43,17 +47,16 @@ class StockMixin(object):
     def _quantity_context(cls, name):
         pool = Pool()
         Date = pool.get('ir.date')
-        today = Date.today()
+
         context = Transaction().context
         new_context = {}
-        stock_date_end = context.get('stock_date_end')
         if name == 'quantity':
-            new_context['forecast'] = False
-            if (stock_date_end or datetime.date.max) > today:
-                new_context['stock_date_end'] = today
+            if (context.get('stock_date_end')
+                    and context['stock_date_end'] > Date.today()):
+                new_context['stock_date_end'] = Date.today()
         elif name == 'forecast_quantity':
             new_context['forecast'] = True
-            if not stock_date_end:
+            if not context.get('stock_date_end'):
                 new_context['stock_date_end'] = datetime.date.max
         return new_context
 
@@ -142,8 +145,7 @@ class StockMixin(object):
             }.get(operator_, lambda v, l: False)
         record_ids = []
         for key, quantity in pbl.items():
-            if (quantity is not None and operand is not None
-                    and operator_(quantity, operand)):
+            if operator_(quantity, operand):
                 # pbl could return None in some keys
                 if key[position] is not None:
                     record_ids.append(key[position])
@@ -163,54 +165,68 @@ class Move(Workflow, ModelSQL, ModelView):
                 ('default_uom_category', '=', Eval('product_uom_category')),
                 ())
             ],
-        depends=DEPENDS + ['product_uom_category'])
+        depends=DEPENDS + ['product_uom_category'],
+        help="The product that the move is associated with.")
     product_uom_category = fields.Function(
         fields.Many2One('product.uom.category', 'Product Uom Category'),
         'on_change_with_product_uom_category')
-    uom = fields.Many2One("product.uom", "Uom", required=True, states=STATES,
+    uom = fields.Many2One("product.uom", "Uom", required=True,
+        states={
+            'readonly': (Eval('state').in_(['cancel', 'assigned', 'done'])
+                | Eval('unit_price')),
+            },
         domain=[
             ('category', '=', Eval('product_uom_category')),
             ],
-        depends=['state', 'product_uom_category'])
+        depends=['state', 'unit_price', 'product_uom_category'],
+        help="The unit in which the quantity is specified.")
     unit_digits = fields.Function(fields.Integer('Unit Digits'),
         'on_change_with_unit_digits')
     quantity = fields.Float("Quantity", required=True,
         digits=(16, Eval('unit_digits', 2)), states=STATES,
-        depends=['state', 'unit_digits'])
+        depends=['state', 'unit_digits'],
+        help="The amount of stock moved.")
     internal_quantity = fields.Float('Internal Quantity', readonly=True,
         required=True)
     from_location = fields.Many2One("stock.location", "From Location",
         select=True, required=True, states=STATES,
-        depends=DEPENDS + LOCATION_DEPENDS, domain=LOCATION_DOMAIN)
+        depends=DEPENDS + LOCATION_DEPENDS, domain=LOCATION_DOMAIN,
+        help="Where the stock is moved from.")
     to_location = fields.Many2One("stock.location", "To Location", select=True,
         required=True, states=STATES,
-        depends=DEPENDS + LOCATION_DEPENDS, domain=LOCATION_DOMAIN)
+        depends=DEPENDS + LOCATION_DEPENDS, domain=LOCATION_DOMAIN,
+        help="Where the stock is moved to.")
     shipment = fields.Reference('Shipment', selection='get_shipment',
-        readonly=True, select=True)
+        readonly=True, select=True,
+        help="Used to group several stock moves together.")
     origin = fields.Reference('Origin', selection='get_origin', select=True,
         states={
             'readonly': Eval('state') != 'draft',
             },
-        depends=['state'])
+        depends=['state'],
+        help="The source of the stock move.")
     planned_date = fields.Date("Planned Date", states={
             'readonly': (Eval('state').in_(['cancel', 'assigned', 'done'])
                 | Eval('shipment'))
             }, depends=['state', 'shipment'],
-        select=True)
+        select=True,
+        help="When the stock is expected to be moved.")
     effective_date = fields.Date("Effective Date", select=True,
         states={
             'required': Eval('state') == 'done',
             'readonly': (Eval('state').in_(['cancel', 'done'])
                 | Eval('shipment')),
             },
-        depends=['state', 'shipment'])
+        depends=['state', 'shipment'],
+        help="When the stock was actually moved.")
     state = fields.Selection([
         ('staging', 'Staging'),
         ('draft', 'Draft'),
         ('assigned', 'Assigned'),
         ('done', 'Done'),
         ('cancel', 'Canceled'),
-        ], 'State', select=True, readonly=True)
+        ], 'State', select=True, readonly=True,
+        help="The current state of the stock move.")
     company = fields.Many2One('company.company', 'Company', required=True,
         states={
             'readonly': Eval('state') != 'draft',
@@ -219,7 +235,8 @@ class Move(Workflow, ModelSQL, ModelView):
             ('id', If(Eval('context', {}).contains('company'), '=', '!='),
                 Eval('context', {}).get('company', -1)),
             ],
-        depends=['state'])
+        depends=['state'],
+        help="The company the stock move is associated with.")
     unit_price = fields.Numeric('Unit Price', digits=price_digits,
         states={
             'invisible': ~Eval('unit_price_required'),
@@ -235,7 +252,8 @@ class Move(Workflow, ModelSQL, ModelView):
             'required': Bool(Eval('unit_price_required')),
             'readonly': Eval('state') != 'draft',
             },
-        depends=['unit_price_required', 'state'])
+        depends=['unit_price_required', 'state'],
+        help="The currency in which the unit price is specified.")
     unit_price_required = fields.Function(
         fields.Boolean('Unit Price Required'),
         'on_change_with_unit_price_required')
@@ -246,7 +264,6 @@ class Move(Workflow, ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Move, cls).__setup__()
-        cls.create_date.select = True
         cls._deny_modify_assigned = set(['product', 'uom', 'quantity',
             'from_location', 'to_location', 'company', 'currency'])
         cls._deny_modify_done_cancel = (cls._deny_modify_assigned |
@@ -256,31 +273,15 @@ class Move(Workflow, ModelSQL, ModelView):
         t = cls.__table__()
         cls._sql_constraints += [
             ('check_move_qty_pos', Check(t, t.quantity >= 0),
-                'Move quantity must be positive'),
+                'stock.msg_move_quantity_positive'),
             ('check_move_internal_qty_pos',
                 Check(t, t.internal_quantity >= 0),
-                'Internal move quantity must be positive'),
+                'stock.msg_move_internal_quantity_positive'),
             ('check_from_to_locations',
                 Check(t, t.from_location != t.to_location),
-                'Source and destination location must be different'),
+                'stock.msg_move_from_to_location'),
             ]
         cls._order[0] = ('id', 'DESC')
-        cls._error_messages.update({
-            'set_state_draft': ('You can not set stock move "%s" to draft '
-                'state.'),
-            'set_state_assigned': ('You can not set stock move "%s" to '
-                'assigned state.'),
-            'set_state_done': 'You can not set stock move "%s" to done state.',
-            'delele_state': ('You can not delete stock move "%s" because '
-                'it is not in staging, draft nor cancelled state.'),
-            'period_closed': ('You can not modify move "%(move)s" because '
-                'period "%(period)s" is closed.'),
-            'modify_assigned': ('You can not modify stock move "%s" because '
-                'it is in "Assigned" state.'),
-            'modify_done_cancel': ('You can not modify stock move "%s" '
-                'because it is in "Done" or "Cancel" state.'),
-            'no_origin': 'The stock moves "%s" have no origin.',
-            })
         cls._transitions |= set((
                 ('staging', 'draft'),
                 ('staging', 'cancel'),
@@ -322,6 +323,9 @@ class Move(Workflow, ModelSQL, ModelView):
         super(Move, cls).__register__(module_name)
         table = cls.__table_handler__(module_name)
 
+        # Add index on create_date
+        table.index_action('create_date', action='add')
+
         # Index for period join in compute_quantities_query
         table.index_action([
                 Coalesce(sql_table.effective_date,
@@ -358,56 +362,19 @@ class Move(Workflow, ModelSQL, ModelView):
             return self.uom.digits
         return 2
 
-    @fields.depends('product', 'currency', 'uom', 'company', 'from_location',
-        'to_location')
+    @fields.depends('product', 'uom')
     def on_change_product(self):
-        pool = Pool()
-        Uom = pool.get('product.uom')
-        Currency = pool.get('currency.currency')
-
-        self.unit_price = Decimal('0.0')
         if self.product:
-            self.uom = self.product.default_uom
-            self.unit_digits = self.product.default_uom.digits
-            unit_price = None
-            if self.from_location and self.from_location.type in ('supplier',
-                    'production'):
-                unit_price = self.product.cost_price
-            elif self.to_location and self.to_location.type == 'customer':
-                unit_price = self.product.list_price
-            if unit_price:
-                if self.uom != self.product.default_uom:
-                    unit_price = Uom.compute_price(self.product.default_uom,
-                        unit_price, self.uom)
-                if self.currency and self.company:
-                    unit_price = Currency.compute(self.company.currency,
-                        unit_price, self.currency, round=False)
-                self.unit_price = unit_price
+            if (not self.uom
+                    or self.uom.category != self.product.default_uom.category):
+                self.uom = self.product.default_uom
+                self.unit_digits = self.product.default_uom.digits
 
     @fields.depends('product')
     def on_change_with_product_uom_category(self, name=None):
         if self.product:
             return self.product.default_uom_category.id
 
-    @fields.depends('product', 'currency', 'uom', 'company', 'from_location',
-        'to_location')
-    def on_change_uom(self):
-        pool = Pool()
-        Uom = pool.get('product.uom')
-        Currency = pool.get('currency.currency')
-
-        self.unit_price = Decimal('0.0')
-        if self.product:
-            if self.to_location and self.to_location.type == 'storage':
-                unit_price = self.product.cost_price
-                if self.uom and self.uom != self.product.default_uom:
-                    unit_price = Uom.compute_price(self.product.default_uom,
-                        unit_price, self.uom)
-                if self.currency and self.company:
-                    unit_price = Currency.compute(self.company.currency,
-                        unit_price, self.currency, round=False)
-                self.unit_price = unit_price
-
     @fields.depends('from_location', 'to_location')
     def on_change_with_unit_price_required(self, name=None):
         from_type = self.from_location.type if self.from_location else None
@@ -480,10 +447,10 @@ class Move(Workflow, ModelSQL, ModelView):
                     date = (move.effective_date if move.effective_date
                         else move.planned_date)
                     if date and date <= period.date:
-                        cls.raise_user_error('period_closed', {
-                                'move': move.rec_name,
-                                'period': period.rec_name,
-                                })
+                        raise AccessError(
+                            gettext('stock.msg_move_modify_period_close',
+                                move=move.rec_name,
+                                period=period.rec_name))
 
     def get_rec_name(self, name):
         return ("%s%s %s"
@@ -543,13 +510,8 @@ class Move(Workflow, ModelSQL, ModelView):
     def set_effective_date(self):
         pool = Pool()
         Date = pool.get('ir.date')
-        ShipmentInternal = pool.get('stock.shipment.internal')
 
-        if (not self.effective_date
-                and isinstance(self.shipment, ShipmentInternal)
-                and self.to_location == self.shipment.transit_location):
-            self.effective_date = self.shipment.effective_start_date
-        elif not self.effective_date and self.shipment:
+        if not self.effective_date and self.shipment:
             self.effective_date = self.shipment.effective_date
         if not self.effective_date:
             self.effective_date = Date.today()
@@ -694,12 +656,15 @@ class Move(Workflow, ModelSQL, ModelView):
             if cls._deny_modify_assigned & vals_set:
                 for move in moves:
                     if move.state == 'assigned':
-                        cls.raise_user_error('modify_assigned', move.rec_name)
+                        raise AccessError(
+                            gettext('stock.msg_move_modify_assigned',
+                                move=move.rec_name))
             if cls._deny_modify_done_cancel & vals_set:
                 for move in moves:
                     if move.state in ('done', 'cancel'):
-                        cls.raise_user_error('modify_done_cancel',
-                            (move.rec_name,))
+                        raise AccessError(
+                            gettext('stock.msg_move_modify_%s' % move.state,
+                                move=move.rec_name))
 
         super(Move, cls).write(*args)
 
@@ -724,7 +689,9 @@ class Move(Workflow, ModelSQL, ModelView):
     def delete(cls, moves):
         for move in moves:
             if move.state not in {'staging', 'draft', 'cancel'}:
-                cls.raise_user_error('delele_state', (move.rec_name,))
+                raise AccessError(
+                    gettext('stock.msg_move_delete_draft_cancel',
+                        move=move.rec_name))
         super(Move, cls).delete(moves)
 
     @staticmethod
@@ -734,6 +701,8 @@ class Move(Workflow, ModelSQL, ModelView):
 
     @classmethod
     def check_origin(cls, moves, types=None):
+        pool = Pool()
+        Warning = pool.get('res.user.warning')
         if types is None:
             types = cls.check_origin_types()
         if not types:
@@ -750,7 +719,10 @@ class Move(Workflow, ModelSQL, ModelView):
                 names += '...'
             warning_name = '%s.done' % hashlib.md5(
                 str(moves).encode('utf-8')).hexdigest()
-            cls.raise_user_warning(warning_name, 'no_origin', names)
+            if Warning.check(warning_name):
+                raise MoveOriginWarning(warning_name,
+                    gettext('stock.msg_move_no_origin',
+                        moves=names))
 
     def pick_product(self, location_quantities):
         """
@@ -805,48 +777,41 @@ class Move(Workflow, ModelSQL, ModelView):
             locations = list(set((m.from_location for m in moves)))
         location_ids = [l.id for l in locations]
         product_ids = list(set((m.product.id for m in moves)))
-        companies = {m.company for m in moves}
         stock_date_end = Date.today()
 
         if database.has_select_for():
-            for company in companies:
-                table = cls.__table__()
-                query = table.select(Literal(1),
-                    where=(table.to_location.in_(location_ids)
-                        | table.from_location.in_(location_ids))
-                    & table.product.in_(product_ids)
-                    & (table.company == company.id),
-                    for_=For('UPDATE', nowait=True))
+            table = cls.__table__()
+            query = table.select(Literal(1),
+                where=(table.to_location.in_(location_ids)
+                    | table.from_location.in_(location_ids))
+                & table.product.in_(product_ids),
+                for_=For('UPDATE', nowait=True))
 
-                PeriodCache = Period.get_cache(grouping)
-                if PeriodCache:
-                    periods = Period.search([
-                            ('date', '<', stock_date_end),
-                            ('company', '=', company.id),
-                            ('state', '=', 'closed'),
-                            ], order=[('date', 'DESC')], limit=1)
-                    if periods:
-                        period, = periods
-                        query.where &= Coalesce(
-                            table.effective_date,
-                            table.planned_date,
-                            datetime.date.max) > period.date
+            PeriodCache = Period.get_cache(grouping)
+            if PeriodCache:
+                periods = Period.search([
+                        ('date', '<', stock_date_end),
+                        ('state', '=', 'closed'),
+                        ], order=[('date', 'DESC')], limit=1)
+                if periods:
+                    period, = periods
+                    query.where &= Coalesce(
+                        table.effective_date,
+                        table.planned_date,
+                        datetime.date.max) > period.date
 
-                with connection.cursor() as cursor:
-                    cursor.execute(*query)
+            with connection.cursor() as cursor:
+                cursor.execute(*query)
         else:
             database.lock(connection, cls._table)
 
-        pblc = {}
-        for company in companies:
-            with Transaction().set_context(
-                    stock_date_end=stock_date_end,
-                    stock_assign=True,
-                    company=company.id):
-                pblc[company.id] = Product.products_by_location(
-                    location_ids,
-                    grouping=grouping,
-                    grouping_filter=(product_ids,))
+        with Transaction().set_context(
+                stock_date_end=stock_date_end,
+                stock_assign=True):
+            pbl = Product.products_by_location(
+                location_ids,
+                grouping=grouping,
+                grouping_filter=(product_ids,))
 
         def get_key(move, location):
             key = (location.id,)
@@ -866,7 +831,6 @@ class Move(Workflow, ModelSQL, ModelView):
                 if move.state == 'staging':
                     success = False
                 continue
-            pbl = pblc[move.company.id]
             # Keep location order for pick_product
             location_qties = OrderedDict()
             if with_childs:
@@ -874,6 +838,7 @@ class Move(Workflow, ModelSQL, ModelView):
                 if childs is None:
                     childs = Location.search([
                             ('parent', 'child_of', [move.from_location.id]),
+                            ('type', '!=', 'view'),
                             ])
                     child_locations[move.from_location] = childs
             else:
@@ -886,6 +851,11 @@ class Move(Workflow, ModelSQL, ModelView):
                         round=False)
             # Prevent to pick from the destination location
             location_qties.pop(move.to_location, None)
+            try:
+                # Try first to pick from source location
+                location_qties.move_to_end(move.from_location, last=False)
+            except KeyError:
+                pass
 
             to_pick = move.pick_product(location_qties)
 
@@ -960,7 +930,7 @@ class Move(Workflow, ModelSQL, ModelView):
             grouping, and the last column is the quantity.
         """
         pool = Pool()
-        User = pool.get('res.user')
+        Rule = pool.get('ir.rule')
         Location = pool.get('stock.location')
         Date = pool.get('ir.date')
         Period = pool.get('stock.period')
@@ -987,7 +957,7 @@ class Move(Workflow, ModelSQL, ModelView):
         assert grouping_filter is None or len(grouping_filter) <= len(grouping)
         assert len(set(grouping)) == len(grouping)
 
-        company = User(Transaction().user).company
+        move_rule_query = Rule.query_get('stock.move')
 
         def get_column(name, table, product):
             if name.startswith('product.'):
@@ -999,7 +969,7 @@ class Move(Workflow, ModelSQL, ModelView):
         if use_product:
             product = Product.__table__()
             columns = ['id', 'state', 'effective_date', 'planned_date',
-            'internal_quantity', 'from_location', 'to_location', 'company']
+            'internal_quantity', 'from_location', 'to_location']
             columns += [c for c in grouping if c not in columns]
             columns = [get_column(c, move, product) for c in columns]
             move = (move
@@ -1010,7 +980,6 @@ class Move(Workflow, ModelSQL, ModelView):
         period = None
         if PeriodCache:
             period_cache = PeriodCache.__table__()
-            period_table = Period.__table__()
             if use_product:
                 product_cache = Product.__table__()
                 columns = ['internal_quantity', 'period', 'location']
@@ -1019,11 +988,7 @@ class Move(Workflow, ModelSQL, ModelView):
                     for c in columns]
                 period_cache = (period_cache
                     .join(product_cache,
-                        condition=(period_cache.product == product_cache.id))
-                    .join(period_table,
-                        condition=(period_cache.period == period_table.id)
-                        & ((period_table.company == company.id)
-                            if company else Literal(True)))
+                        condition=period_cache.product == product_cache.id)
                     .select(*columns))
 
         if with_childs:
@@ -1034,7 +999,7 @@ class Move(Workflow, ModelSQL, ModelView):
             to_location = Location.__table__()
             to_parent_location = Location.__table__()
             columns = ['id', 'state', 'effective_date', 'planned_date',
-                'internal_quantity', 'company']
+                'internal_quantity']
             columns += [c for c in grouping if c not in columns]
             columns = [Column(move, c).as_(c) for c in columns]
 
@@ -1235,7 +1200,6 @@ class Move(Workflow, ModelSQL, ModelView):
             periods = Period.search([
                     ('date', '<=', context['stock_date_end']),
                     ('state', '=', 'closed'),
-                    ('company', '=', company.id if company else -1),
                     ], order=[('date', 'DESC')], limit=1)
             if periods:
                 period, = periods
@@ -1293,7 +1257,8 @@ class Move(Workflow, ModelSQL, ModelView):
             where=state_date_clause_in
             & where
             & move.to_location.in_(location_query)
-            & ((move.company == company.id) if company else Literal(True))
+            & (move.id.in_(move_rule_query) if move_rule_query
+                else Literal(True))
             & dest_clause_from,
             group_by=[move.to_location] + move_keys)
         query = Union(query, move.select(move.from_location.as_('location'),
@@ -1302,7 +1267,8 @@ class Move(Workflow, ModelSQL, ModelView):
                 where=state_date_clause_out
                 & where
                 & move.from_location.in_(location_query)
-                & ((move.company == company.id) if company else Literal(True))
+                & (move.id.in_(move_rule_query) if move_rule_query
+                    else Literal(True))
                 & dest_clause_to,
                 group_by=[move.from_location] + move_keys),
             all_=True)
@@ -1360,7 +1326,7 @@ class Move(Workflow, ModelSQL, ModelView):
             Model = Template
         id_getter = operator.itemgetter(grouping.index(id_name) + 1)
         ids = set()
-        quantities = defaultdict(lambda: 0)
+        quantities = defaultdict(int)
         keys = set()
         # We can do a quick loop without propagation if the request is for a
         # single location because all the locations are children and we can sum
