@@ -231,32 +231,17 @@ class StockTestCase(ModuleTestCase):
                 'forecast': True,
                 }, 6),
             ]
-            today_quantity = 4
 
             def tests_product_quantity(context, quantity):
                 with transaction.set_context(locations=[storage.id]):
                     product_reloaded = Product(product.id)
                     if (not context.get('stock_date_end')
-                            or context['stock_date_end'] > today):
-                        self.assertEqual(
-                            product_reloaded.forecast_quantity, quantity,
-                            msg='context %r' % context)
-                        self.assertEqual(
-                            product_reloaded.quantity, today_quantity,
-                            msg='context %r' % context)
-                    elif context.get('forecast'):
-                        self.assertEqual(
-                            product_reloaded.forecast_quantity, quantity,
-                            msg='context %r' % context)
-                    elif context.get('stock_date_end') == today:
-                        self.assertEqual(product_reloaded.quantity, quantity,
-                            msg='context %r' % context)
+                            or context['stock_date_end'] > today
+                            or context.get('forecast')):
+                        self.assertEqual(product_reloaded.forecast_quantity,
+                            quantity)
                     else:
-                        self.assertEqual(
-                            product_reloaded.forecast_quantity, quantity,
-                            msg='context %r' % context)
-                        self.assertEqual(product_reloaded.quantity, quantity,
-                            msg='context %r' % context)
+                        self.assertEqual(product_reloaded.quantity, quantity)
 
             def tests_product_search_quantity(context, quantity):
                 with transaction.set_context(locations=[storage.id]):
@@ -1090,6 +1075,126 @@ class StockTestCase(ModuleTestCase):
             self.assertFalse(Move.assign_try(moves))
             self.assertEqual([m.state for m in moves], ['assigned', 'draft'])
 
+    @with_transaction()
+    def test_assign_try_skip_to_location(self):
+        "Test Move assign_try skip to_location"
+        pool = Pool()
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Uom = pool.get('product.uom')
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+
+        uom, = Uom.search([('name', '=', 'Meter')])
+        template = Template(
+            name='Test Move.assign_try',
+            type='goods',
+            list_price=Decimal(1),
+            default_uom=uom,
+            )
+        template.save()
+        product = Product(template=template.id)
+        product.save()
+
+        supplier, = Location.search([('code', '=', 'SUP')])
+        storage, = Location.search([('code', '=', 'STO')])
+        child, = Location.copy([storage], default={'parent': storage.id})
+
+        company = create_company()
+        with set_company(company):
+            move, = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': supplier.id,
+                        'to_location': child.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+            Move.do([move])
+
+            move, = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': storage.id,
+                        'to_location': child.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+
+            self.assertFalse(Move.assign_try([move]))
+            self.assertEqual(move.state, 'draft')
+
+    @with_transaction()
+    def test_assign_try_prefer_from_location(self):
+        "Test Move assign_try prefer from location"
+        pool = Pool()
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Uom = pool.get('product.uom')
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+
+        uom, = Uom.search([('name', '=', 'Meter')])
+        template = Template(
+            name='Test Move.assign_try',
+            type='goods',
+            list_price=Decimal(1),
+            default_uom=uom,
+            )
+        template.save()
+        product = Product(template=template.id)
+        product.save()
+
+        supplier, = Location.search([('code', '=', 'SUP')])
+        storage, = Location.search([('code', '=', 'STO')])
+        # Ensure storage2 comes first when ordering locations by name
+        storage2, = Location.copy(
+            [storage], default={'name': "AAA", 'parent': storage.id})
+        customer, = Location.search([('code', '=', 'CUS')])
+
+        company = create_company()
+        with set_company(company):
+            moves = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': supplier.id,
+                        'to_location': storage.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }, {
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': supplier.id,
+                        'to_location': storage2.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+            Move.do(moves)
+
+            move, = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': storage.id,
+                        'to_location': customer.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+
+            Move.assign_try([move])
+
+            self.assertEqual(move.state, 'assigned')
+            self.assertEqual(move.from_location, storage)
+
     @with_transaction()
     def test_assign_without_moves(self):
         "Test Move assign_try with empty moves"
@@ -1229,7 +1334,6 @@ class StockTestCase(ModuleTestCase):
                     'name': "Product",
                     'type': 'goods',
                     'list_price': Decimal(0),
-                    'cost_price': Decimal(0),
                     'default_uom': unit.id,
                     }])
         product, = Product.create([{'template': template.id}])
@@ -1270,7 +1374,6 @@ class StockTestCase(ModuleTestCase):
                     'name': "Product",
                     'type': 'goods',
                     'list_price': Decimal(0),
-                    'cost_price': Decimal(0),
                     'default_uom': unit.id,
                     }])
         product, = Product.create([{'template': template.id}])
